//class PSWCommitter
function PSWCommitter()
{
	this.hdrs = [];
	this.delHdrs = Components.classes["@mozilla.org/array;1"]
                        .createInstance(Components.interfaces.nsIMutableArray);
	this.newKey = null;
	this.tempMsgs = [];
	this.pref = new PSWPref();
	this.delRemaining = 0;
}

PSWCommitter.prototype.append = function(msgHdr)
{
	this.hdrs.push(msgHdr);
	this.delHdrs.appendElement(msgHdr, false);
}

PSWCommitter.prototype.loadMessageToString = function(msgHdr)
{
	var dbView = this.parentWindow ? this.parentWindow.gDBView : gDBView;
	var IOService = Components.classes['@mozilla.org/network/io-service;1'].
	  getService(Components.interfaces.nsIIOService);
	var msg = "";
	try {
		var uri = msgHdr.folder.getUriForMsg(msgHdr);
		var messageService = messenger.messageServiceFromURI(uri);
		var messageStream = Components.classes["@mozilla.org/network/sync-stream-listener;1"].
		  createInstance().QueryInterface(Components.interfaces.nsIInputStream);
		var inputStream = Components.classes["@mozilla.org/scriptableinputstream;1"].
		  createInstance().QueryInterface(Components.interfaces.nsIScriptableInputStream);
		inputStream.init(messageStream);
		messageService.streamMessage(uri, messageStream, msgWindow, null, false, null);

		//remember the last priority in the header
		while (inputStream.available()) {
			msg += inputStream.read(512);
		}
		messageStream.close();
		inputStream.close();
	} catch(e) {
		return null;
	}

	var headerEnd = -1;
	var delimiter = null;
	var candidates = ["\r\n", "\n\r", "\n", "\r"];
	var i;
	for (i=0; i<candidates.length; i++) {
		var candidate = candidates[i];
		headerEnd = msg.indexOf(candidate + candidate);
		if (headerEnd != -1) {
			headerEnd += candidate.length;
			break;		
		}
	}

	if (headerEnd != -1) {
		return [msg.substring(0,headerEnd), msg.substring(headerEnd, msg.length), candidates[i]];
	} else {
		return null;
	}
}

PSWCommitter.prototype.makeHdrList = function(hdrString, delimiter)
{
	var hdrs = hdrString.split(delimiter);
	var list = [];
	var keys = [];
	
	var prevKey = "";
	var hdrIndex = -1;
	for (var i=0; i<hdrs.length; i++) {
		var hdr = hdrs[i];
		if (hdr.match(/^(\S+:|From \-)\s+(.+)$/)) {
			hdrIndex++;
			list[hdrIndex] = {key: RegExp.$1, value: RegExp.$2};
			keys.push(RegExp.$1);
			prevKey = RegExp.$1;
		} else {
			if (hdr != "") list[hdrIndex].value += delimiter + hdr;
		}
	}

	return {keys: keys, list: list};
}

PSWCommitter.prototype.getMessageHeaderIndex = function(keys, key)
{
	var messageIDIndex = -1;
	for (var i=0; i<keys.length; i++) {
		var key1 = keys[i];
		if (key1.toLowerCase() == key.toLowerCase()) {
			messageIDIndex = i;
			break;
		}
	}
	
	return messageIDIndex;
}

PSWCommitter.prototype.makeTempMsgFile = function(msgHdr, newPriority, server)
{
	var msg = this.loadMessageToString(msgHdr);
	if (!msg) return null;

	var priority = PSWMsgPriorityValue.convertToXPriorityValue(newPriority);
	var hdrs = this.makeHdrList(msg[0], msg[2]);

	//add Date header if it is missing
	var fromDashIndex = this.getMessageHeaderIndex(hdrs.keys, "From -");
	if (this.getMessageHeaderIndex(hdrs.keys, "Date:") == -1 && fromDashIndex != -1) {
		hdrs.list.push({key: "Date:", value: hdrs.list[fromDashIndex].value});
	}

	//modify Date: to edit a message on IMAP
	/*
	var dateIndex = hdrs.keys.indexOf("Date:");
	if (server == "imap" && dateIndex != -1 && hdrs.list[dateIndex].value) {
		var dateObj = hdrs.list[dateIndex];
		var d = new Date(dateObj.value);
		var d2 = new Date(d.getTime()+1000);
		dateObj.value = d2.toUTCString();
	}
	*/
	var orgPriority = "none";
	var xPriorityIndex = this.getMessageHeaderIndex(hdrs.keys, "X-Priority:");
	if (xPriorityIndex == -1) {
		hdrs.list.push({key: "X-Priority:", value: priority});
	} else {
		orgPriority = hdrs.list[xPriorityIndex].value;
		hdrs.list[xPriorityIndex].value = priority;		
	}
	
	if (hdrs.keys.indexOf("X-PSW-Org:") == -1) {
		hdrs.list.push({key: "X-PSW-Org:", value: orgPriority});
	}

	//change Message-ID. It seems Gmail IMAP cannot replace if the ID is the same.
	var messageIDIndex = this.getMessageHeaderIndex(hdrs.keys, "message-id:");
	if (messageIDIndex != -1 && this.pref.changeMessageIdOnIMAPCommit(server)) {
		var messageID = hdrs.list[messageIDIndex].value;
		hdrs.list[messageIDIndex].value = messageID.replace(/\@/,".psw@");
	}
	
	var newHdrStr = "";
	for (var i=0; i<hdrs.list.length; i++) {
		var hdrObj = hdrs.list[i];
		if (hdrObj.value !== null) newHdrStr += hdrObj.key + " " + hdrObj.value + msg[2];
	}
	msg[0] = newHdrStr;

	var tempFileNativePath = null;
	try {
		var dirService = Components.classes["@mozilla.org/file/directory_service;1"].
		  getService(Components.interfaces.nsIProperties);
		var tempDir = dirService.get("TmpD", Components.interfaces.nsIFile);
		var separator = "/";
		if (navigator.platform.toLowerCase().indexOf("win") != -1) {
            separator = '\\';
        }
		tempFileNativePath = tempDir.path + separator + "psw_" + msgHdr.messageKey + ".eml";
		var sfile = Components.classes["@mozilla.org/file/local;1"].
		  createInstance(Components.interfaces.nsILocalFile);
		sfile.initWithPath(tempFileNativePath);
		if (sfile.exists()) sfile.remove(true);
		sfile.create(sfile.NORMAL_FILE_TYPE, 0666);
		var stream = Components.classes['@mozilla.org/network/file-output-stream;1'].
		  createInstance(Components.interfaces.nsIFileOutputStream);
		stream.init(sfile, 2, 0x200, false); // open as "write only"
		stream.write(msg[0]+msg[1], (msg[0]+msg[1]).length);
		stream.close();
	} catch(e) {
		dump(e+"\n");
		return null;
	}

	return tempFileNativePath;
}

PSWCommitter.prototype.correctMsgHdr = function()
{
	var msgHdr;
	try {
		msgHdr = this.tempMsgs[0].folder.GetMessageHeader(this.newKey);
	} catch(e) {
		//wait until msgHdr is available
		var me = this;
		var func = function() {
			me.correctMsgHdr();
		}
		setTimeout(func, 1000);
		return;
	}
	
	//remove temp file
	var msgObj = this.tempMsgs.shift();
	if (msgObj.file.exists()) msgObj.file.remove(true);
	
	msgHdr.setStringProperty("junkscore",msgObj.junkscore);
	msgHdr.setStringProperty("label",msgObj.label);
	msgHdr.setStringProperty("keywords",msgObj.keywords);

	this.copyTempFileMessages();
}

PSWCommitter.prototype.commit = function()
{
	if (this.hdrs.length == 0) {
		return;
	}
	for (var i=0; i<this.hdrs.length; i++) {
		var msgHdr = this.hdrs[i];
		var folder = msgHdr.folder;
		var server = folder.server.type;
		var hostName = folder.hostname;
		//RSS and News etc... are not supported
		if (server != "pop3" && server != "imap" && server != "none") continue;
		/*
		if (hostName.indexOf("imap.gmail.com")!= -1 || hostName.indexOf("imap.googlemail.com")!= -1) {
			dump("Commiting on Gmail IMAP is not supported.\n")
			continue;
		}
		*/
		var dbView = this.parentWindow ? this.parentWindow.gDBView : gDBView;
		var key = msgHdr.messageKey;
		var tmpFilePath = this.makeTempMsgFile(msgHdr, msgHdr.priority, folder.server);
		if (!tmpFilePath) continue;
		var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
	  file.initWithPath(tmpFilePath);
  	file.QueryInterface(Components.interfaces.nsIFile);
		this.tempMsgs.push(
											 {file: file,
											  folder: folder,
											  msgHdr: msgHdr,
											  flags: msgHdr.flags,
											  junkscore: msgHdr.getStringProperty("junkscore"),
		 										label: msgHdr.label,
		 										keywords: msgHdr.getStringProperty("keywords")}
		);
	}
	this.deleteOriginalMessages();
}

PSWCommitter.prototype.deleteOriginalMessages = function()
{
	if (this.tempMsgs.length > 0) {
		var folder = this.tempMsgs[0].folder;
		var deleteStorage = !this.pref.saveOrgInTrashOnCommit();
		this.delHdrs.QueryInterface(Components.interfaces.nsIArray);
		this.delRemaining = this.delHdrs.length;
	  Components.classes["@mozilla.org/messenger/services/session;1"]
	      	.getService(Components.interfaces.nsIMsgMailSession)
	        .AddFolderListener(this, Components.interfaces.nsIFolderListener.all);
		folder.deleteMessages(this.delHdrs, msgWindow, deleteStorage, true, null, false);
	}
}

PSWCommitter.prototype.copyTempFileMessages = function()
{
	if (this.tempMsgs.length > 0) {
		var msgObj = this.tempMsgs[0];
	  msgObj.folder.copyFileMessage(msgObj.file, null, false, msgObj.flags, msgObj.keywords, msgWindow, this);
	}
}

PSWCommitter.prototype.OnProgress = function(progress, progressMax)
{
}

PSWCommitter.prototype.OnStartCopy = function()
{
}

PSWCommitter.prototype.OnStopCopy = function(status)
{
}

PSWCommitter.prototype.SetMessageKey = function(key)
{
	this.newKey = key;
	var me = this;
	var func = function() {
		me.correctMsgHdr();
	};
	setTimeout(func, 100);
}

PSWCommitter.prototype.OnItemAdded = function(parentItem, item, view)
{
}

PSWCommitter.prototype.OnItemRemoved = function(parentItem, item, view)
{
	this.delRemaining--;
	if (this.delRemaining == 0) {
	  Components.classes["@mozilla.org/messenger/services/session;1"]
	      	.getService(Components.interfaces.nsIMsgMailSession)
	        .RemoveFolderListener(this);
		this.copyTempFileMessages();
	}
}

PSWCommitter.prototype.OnItemPropertyChanged = function(item, property, oldValue, newValue)
{
}

PSWCommitter.prototype.OnItemIntPropertyChanged = function(item, property, oldValue, newValue)
{
}

PSWCommitter.prototype.OnItemBoolPropertyChanged = function(item, property, oldValue, newValue)
{
}

PSWCommitter.prototype.OnItemUnicharPropertyChanged = function(item, property, oldValue, newValue)
{
}
PSWCommitter.prototype.OnItemPropertyFlagChanged = function(item, property, oldFlag, newFlag)
{
}

PSWCommitter.prototype.OnItemEvent = function(folder, event)
{
}
